@@ -149,6 +149,8 @@ class WorkerManager(object):
             command.extend(
                 ['--checkin-frequency-seconds', str(self.args.worker_checkin_frequency_seconds)]
             )
+        if self.args.worker_shared_memory_size_gb:
+            command.extend(['--shared-memory-size-gb', str(self.args.worker_shared_memory_size_gb)])
 
         return command
 
